#ifndef FEDATASTRUCTURES_HEADER
#define FEDATASTRUCTURES_HEADER

#include <stdint.h> // for fixed-with types.

typedef struct Grid {
  uint64_t NumberOfPoints;
  uint64_t NumberOfCells;
  double* Points;
  unsigned int numPoints[3];
  unsigned int Extent[6];
  double Spacing[3];
} Grid;

void InitializeGrid(Grid* grid, const unsigned int numPoints[3], const double spacing[3]);
void FinalizeGrid(Grid*);

typedef struct Attributes {
  // A structure for generating and storing point and cell fields.
  // Velocity is stored at the points and density is stored
  // for the cells. The current velocity profile is for a
  // shearing flow with U(y,t) = y*t, V = 0 and W = 0.
  // Density is constant through the domain.
  double* Velocity;
  float* Density;
  Grid* GridPtr;
} Attributes;

void InitializeAttributes(Attributes* attributes, Grid* grid);
void FinalizeAttributes(Attributes* attributes);
void UpdateFields(Attributes* attributes, double time);
#endif
